@@ -47,17 +47,13 @@
 // Bottle graphics
 #include "msgInBottle.c"
 
-static int width;
-static int height;
-static GtkWidget *drawingArea = NULL;
-
-static guchar *rgb_buffer;
-
 static void
 display_about_dialog (BonoboUIComponent *uic,
-		      BubblemonApplet *bubble,
-		      const gchar       *verbname)
+		      gpointer data,
+		      const gchar *verbname)
 {
+  BubblemonApplet *bubble = (BubblemonApplet*)data;
+
   static const gchar *authors[] = { "Johan Walles <johan.walles@gmail.com>",
 				    "Juan Salaverria <rael@vectorstar.net>",
 				    NULL };
@@ -86,7 +82,7 @@ display_about_dialog (BonoboUIComponent *uic,
 }
 
 static void
-ui_update (void)
+ui_update (BubblemonApplet *applet)
 {
   int w, h, i;
   const bubblemon_picture_t *bubblePic;
@@ -95,15 +91,17 @@ ui_update (void)
 
   GdkGC *gc;
 
+  GtkWidget *drawingArea = applet->drawingArea;
+
   if((drawingArea == NULL) ||
      !GTK_WIDGET_REALIZED(drawingArea) ||
      !GTK_WIDGET_DRAWABLE(drawingArea) ||
-     width <= 0)
+     applet->width <= 0)
   {
     return;
   }
 
-  bubblePic = bubblemon_getPicture();
+  bubblePic = bubblemon_getPicture(applet->bubblemon);
   if ((bubblePic == NULL) ||
       (bubblePic->width == 0) ||
       (bubblePic->pixels == 0))
@@ -115,7 +113,7 @@ ui_update (void)
 
   gc = gdk_gc_new(drawingArea->window);
 
-  p = rgb_buffer;
+  p = applet->rgb_buffer;
   pixel = bubblePic->pixels;
   for(i = 0; i < w * h; i++) {
     *(p++) = pixel->components.r;
@@ -125,41 +123,40 @@ ui_update (void)
   }
 
   gdk_draw_rgb_image(drawingArea->window, gc,
-                     0, 0, width, height,
+                     0, 0,
+		     applet->width, applet->height,
                      GDK_RGB_DITHER_NORMAL,
-                     rgb_buffer, w * 3);
+                     applet->rgb_buffer, w * 3);
 
   gdk_gc_destroy(gc);
 
 }
 
 static int
-ui_expose (void)
+ui_expose(GtkWidget *exposed, GdkEventExpose *event, gpointer data)
 {
-  ui_update();
+  BubblemonApplet *applet = (BubblemonApplet*)data;
+
+  ui_update(applet);
   return FALSE;
 }
 
 static int
-ui_timeoutHandler(gpointer ignored)
+ui_realize(GtkWidget *realized, gpointer data)
 {
+  BubblemonApplet *applet = (BubblemonApplet*)data;
 
-  ui_update();
-  return TRUE;
+  ui_update(applet);
+  return FALSE;
 }
 
-static void
-set_tooltip (gpointer bubbles)
+static int
+ui_timeoutHandler(gpointer data)
 {
-  GtkTooltips *tooltips;
-  BubblemonApplet *bubble = bubbles;
+  BubblemonApplet *applet = (BubblemonApplet*)data;
 
-  tooltips = gtk_tooltips_new ();
-  g_object_ref (tooltips);
-  gtk_object_sink (GTK_OBJECT (tooltips));
-  g_object_set_data (G_OBJECT (bubble->applet), "tooltips", tooltips);
-  gtk_tooltips_set_tip (tooltips, bubble->applet, bubblemon_getTooltip(), NULL);
-  bubble->tooltips = tooltips;
+  ui_update(applet);
+  return TRUE;
 }
 
 static int
@@ -167,51 +164,48 @@ update_tooltip (gpointer bubbles)
 {
   BubblemonApplet *bubble = bubbles;
 
-  gtk_tooltips_set_tip (bubble->tooltips, bubble->applet, bubblemon_getTooltip(), NULL);
+  gtk_widget_set_tooltip_text(bubble->applet,
+			      bubblemon_getTooltip(bubble->bubblemon));
+
+  // FIXME: We want to call
+  // gtk_widget_trigger_tooltip_query(bubble->applet) here, but we can't
+  // due to http://bugs.debian.org/510873.
+  //
+  // There may be other problems as well, try running the applet in
+  // valgrind after 510873 is fixed and see if we can show the tooltip
+  // without any complaints from valgrind then.
+
   return TRUE;
 }
 
-
-void
-destroy_tooltip (GtkWidget *object)
-{
-  GtkTooltips *tooltips;
-
-  tooltips = g_object_get_data (G_OBJECT (object), "tooltips");
-  if (tooltips) {
-    g_object_unref (tooltips);
-    g_object_set_data (G_OBJECT (object), "tooltips", NULL);
-  }
-}
-
-
 static void
-applet_destroy (GtkWidget *applet, BubblemonApplet *bubble)
+applet_destroy (GtkWidget *panelApplet, BubblemonApplet *applet)
 {
-  destroy_tooltip (GTK_WIDGET (bubble->applet));
+  g_source_remove(applet->refresh_timeout_id);
+  applet->refresh_timeout_id = 0;
+  g_source_remove(applet->tooltip_timeout_id);
+  applet->tooltip_timeout_id = 0;
 
-  if (bubble->aboutbox != NULL)
-    gtk_widget_destroy(bubble->aboutbox);
-  bubble->aboutbox = NULL;
+  if (applet->aboutbox != NULL) {
+    gtk_widget_destroy(applet->aboutbox);
+  }
+  applet->aboutbox = NULL;
 
-  g_free(bubble);
+  if (applet->rgb_buffer != NULL) {
+    g_free(applet->rgb_buffer);
+  }
+  applet->rgb_buffer = NULL;
 
-  bubblemon_done();
+  bubblemon_done(applet->bubblemon);
+
+  g_free(applet);
 }
 
 static gboolean
-applet_reconfigure (GtkDrawingArea *drawingArea, GdkEventConfigure *event, gpointer data)
+applet_reconfigure (GtkDrawingArea *drawingArea, GdkEventConfigure *event, BubblemonApplet *bubble)
 {
-  BubblemonApplet *bubble = (BubblemonApplet *)data;
-
-  if (bubble->width == event->width
-    && bubble->height == event->height)
-  {
-    return TRUE;
-  }
-
-  width = event->width;
-  height = event->height;
+  int width = event->width;
+  int height = event->height;
 
   PanelAppletOrient orientation =
     panel_applet_get_orient(PANEL_APPLET(bubble->applet));
@@ -237,6 +231,7 @@ applet_reconfigure (GtkDrawingArea *drawingArea, GdkEventConfigure *event, gpoin
   if (bubble->width == width
       && bubble->height == height)
   {
+    // Already at the correct size, done!
     return TRUE;
   }
 
@@ -245,34 +240,31 @@ applet_reconfigure (GtkDrawingArea *drawingArea, GdkEventConfigure *event, gpoin
   bubble->width = width;
   bubble->height = height;
 
-  /* not yet all loaded up */
-  if (bubble->frame == NULL) {
+  if (bubble->applet == NULL) {
+    // Not yet all loaded up
     return TRUE;
   }
 
-  rgb_buffer = realloc(rgb_buffer, width * height * 3);
-  bubblemon_setSize(width, height);
+  bubble->rgb_buffer = g_realloc(bubble->rgb_buffer, width * height * 3);
+  bubblemon_setSize(bubble->bubblemon, width, height);
 
-  ui_update();
+  ui_update(bubble);
 
   return TRUE;
 }
 
 static const BonoboUIVerb bubblemon_menu_verbs [] = {
-  BONOBO_UI_UNSAFE_VERB ("About",       display_about_dialog),
-
+  BONOBO_UI_VERB ("About", display_about_dialog),
   BONOBO_UI_VERB_END
 };
 
 static gboolean
-bubblemon_applet_fill (PanelApplet *applet)
+bubblemon_applet_fill(BubblemonApplet *bubblemon_applet, PanelApplet *applet)
 {
-  BubblemonApplet *bubblemon_applet;
+  GtkWidget *drawingArea;
 
   panel_applet_set_flags(applet, PANEL_APPLET_EXPAND_MINOR);
 
-  bubblemon_applet = g_new0 (BubblemonApplet, 1);
-
   bubblemon_applet->applet = GTK_WIDGET (applet);
   bubblemon_applet->width   = 0;
   bubblemon_applet->height= 0;
@@ -282,11 +274,9 @@ bubblemon_applet_fill (PanelApplet *applet)
 		    G_CALLBACK (applet_destroy),
 		    bubblemon_applet);
 
-  bubblemon_applet->frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (bubblemon_applet->frame), GTK_SHADOW_IN);
-
   drawingArea = gtk_drawing_area_new();
   g_assert(drawingArea != NULL);
+  bubblemon_applet->drawingArea = drawingArea;
   gtk_widget_set_size_request(GTK_WIDGET(drawingArea), RELATIVE_WIDTH, RELATIVE_HEIGHT);
 
   g_signal_connect (G_OBJECT (drawingArea),
@@ -299,23 +289,15 @@ bubblemon_applet_fill (PanelApplet *applet)
 			| GDK_ENTER_NOTIFY_MASK
 			| GDK_STRUCTURE_MASK);
 
-  gtk_container_add(GTK_CONTAINER (bubblemon_applet->frame), drawingArea);
-  gtk_widget_show(drawingArea);
-  gtk_widget_show(bubblemon_applet->frame);
+  gtk_container_add(GTK_CONTAINER (bubblemon_applet->applet), drawingArea);
 
   gtk_signal_connect_after(GTK_OBJECT(drawingArea), "realize",
-			   GTK_SIGNAL_FUNC(ui_update), bubblemon_applet);
+			   GTK_SIGNAL_FUNC(ui_realize), bubblemon_applet);
 
   gtk_signal_connect(GTK_OBJECT(drawingArea), "expose_event",
 		     GTK_SIGNAL_FUNC(ui_expose), bubblemon_applet);
 
-  set_tooltip (bubblemon_applet);
-
-  gtk_container_add (GTK_CONTAINER (bubblemon_applet->applet), bubblemon_applet->frame);
-
-  gtk_widget_show_all (GTK_WIDGET (bubblemon_applet->frame));
-
-  gtk_widget_show (GTK_WIDGET (bubblemon_applet->applet));
+  gtk_widget_show_all (GTK_WIDGET (bubblemon_applet->applet));
 
   panel_applet_setup_menu_from_file (PANEL_APPLET (bubblemon_applet->applet),
 				     NULL,
@@ -324,24 +306,29 @@ bubblemon_applet_fill (PanelApplet *applet)
 				     bubblemon_menu_verbs,
 				     bubblemon_applet);
 
-  gtk_timeout_add(1000 / FRAMERATE, ui_timeoutHandler, NULL);
-  gtk_timeout_add(2000, update_tooltip, bubblemon_applet);
+  bubblemon_applet->refresh_timeout_id =
+    g_timeout_add(1000 / FRAMERATE, ui_timeoutHandler, bubblemon_applet);
+  bubblemon_applet->tooltip_timeout_id =
+    g_timeout_add(2000, update_tooltip, bubblemon_applet);
 
   return TRUE;
 }
 
 static gboolean
-bubble_applet_factory (PanelApplet *applet,
+bubble_applet_factory (PanelApplet *panel_applet,
 		       const gchar *iid,
 		       gpointer     data)
 {
+  BubblemonApplet *bubblemon_applet;
+
   gboolean retval = FALSE;
 
-  // Initialize the load metering
-  bubblemon_init();
+  bubblemon_applet = g_new0(BubblemonApplet, 1);
+  bubblemon_applet->bubblemon = bubblemon_init();
 
-  if (strcmp(iid, "OAFIID:GNOME_BubblemonApplet") == 0)
-    retval = bubblemon_applet_fill (applet);
+  if (strcmp(iid, "OAFIID:GNOME_BubblemonApplet") == 0) {
+    retval = bubblemon_applet_fill(bubblemon_applet, panel_applet);
+  }
 
   return retval;
 }
