@@ -1,6 +1,6 @@
 /*
  *  Bubbling Load Monitoring Applet
- *  Copyright (C) 1999-2004, 2008 Johan Walles - johan.walles@gmail.com
+ *  Copyright (C) 1999-2004, 2008, 2009 Johan Walles - johan.walles@gmail.com
  *  http://www.nongnu.org/bubblemon/
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,20 +25,29 @@
 #include <panel-applet.h>
 #include <sys/types.h>
 
+#include "bubblemon.h"
+
 #define FRAMERATE 25
 
 typedef struct
 {
-  GtkWidget         *applet;
-  GtkWidget         *frame;
-  GdkPixmap         *pix;
+  GtkWidget *applet;
+  GtkWidget *drawingArea;
+  GdkPixmap *pix;
+
+  guchar *rgb_buffer;
 
   int width;
   int height;
 
-  GtkWidget         *aboutbox;
-  GtkTooltips       *tooltips;
+  GtkWidget *aboutbox;
 
+  guint refresh_timeout_id;
+  guint tooltip_timeout_id;
+
+  // Reference to our load-metering, bubble-drawing, tool-tipping
+  // bubblemon thingy.
+  bubblemon_t *bubblemon;
 } BubblemonApplet;
 
 #endif
